'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (str, options) {
  var parts = (str || '').split(/(\S+\s+)/).filter(function (s) {
    return s;
  });
  var node = void 0;
  var column = 0;
  var columnEnd = 0;
  var nodes = [];
  parts.forEach(function (v) {
    var trailingSpace = v.match(/\s+$/);
    trailingSpace = trailingSpace && trailingSpace[0];
    if (isLogicalIf(v)) {
      var _v$trim$split = v.trim().split(':'),
          _v$trim$split2 = _toArray(_v$trim$split),
          pathStr = _v$trim$split2[0],
          values = _v$trim$split2.slice(1);

      var isUnless = values[0] === '';
      if (isUnless) {
        values.splice(0, 1); // remove leading blank that is not used in unless
      }
      var helperName = isUnless ? "unless" : "if";
      var startMust = column;
      column += 2; // "{{"
      var startIf = column;
      column += helperName.length;
      var endIf = column;
      // " " between {{if thing "then" "else"}}
      //                 ^
      column = column + 1; // " "
      var startPath = column;
      column += pathStr.length;
      var endPath = column;
      var path = _glimmerSyntax.builders.path(pathStr, _glimmerSyntax.builders.loc(1, startPath, 1, endPath));

      var params = values.map(function (v, i) {
        var str = _glimmerSyntax.builders.string(v);
        var length = v.length;
        if (i < values.length) {
          // leading " " before {{if thing "then" "else"}}
          //                              ^      ^
          column = column + 1;
        }
        var startStr = column;
        column += length + 2; // plus 2 quotes
        var endStr = column;

        str.loc = _glimmerSyntax.builders.loc(1, startStr, 1, endStr);

        return str;
      });

      column += 2; // "}}"
      var endMust = column;

      node = _glimmerSyntax.builders.mustache(helperName, [path].concat(_toConsumableArray(params)), null, null, _glimmerSyntax.builders.loc(1, startMust, 1, endMust));

      // "if" or "unless" location
      node.path.loc = _glimmerSyntax.builders.loc(1, startIf, 1, endIf);

      nodes.push(node);

      if (trailingSpace) {
        column = pushText(trailingSpace, column, nodes, options);
      }
    } else if (isStatic(v)) {
      var value = v.replace(/^:/, '');
      column = pushText(value, column, nodes, options);
    } else {
      column = pushBound(v.trim(), column, nodes, options);
      if (trailingSpace) {
        column = pushText(trailingSpace, column, nodes, options);
      }
    }
  });
  return nodes;
};

var _glimmerSyntax = require('glimmer-engine/dist/node_modules/glimmer-syntax');

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _repeat = require('./utils/repeat');

var _repeat2 = _interopRequireDefault(_repeat);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function isLogicalIf(str) {
  return (/(.+:.+:.+|.+:.+)/.test(str)
  );
}

function isStatic(str) {
  return (/:.+/.test(str)
  );
}

function pushBound(value, column, nodes, options) {
  var columnEnd = column + value.length + 4; // {{}} = 4
  var node = _glimmerSyntax.builders.mustache(value);
  node.loc = _glimmerSyntax.builders.loc(1, column, 1, columnEnd);
  node.path.loc = _glimmerSyntax.builders.loc(1, column + 2, 1, columnEnd - 2);
  nodes.push(node);
  return columnEnd;
}

function pushText(value, column, nodes, options) {
  var columnEnd = column + value.length;
  var node = _glimmerSyntax.builders.text(value);
  node.loc = _glimmerSyntax.builders.loc(1, column, 1, columnEnd);
  nodes.push(node);
  return columnEnd;
}