'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.removeBindAttr = exports.attributeBindingToAttribute = undefined;
exports.default = convertBindAttr;

var _glimmerSyntax = require('glimmer-engine/dist/node_modules/glimmer-syntax');

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _location = require('../utils/location');

var _node = require('../utils/node');

var _classStringParser = require('../class-string-parser');

var _classStringParser2 = _interopRequireDefault(_classStringParser);

var _printer = require('../printer');

var _printer2 = _interopRequireDefault(_printer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isBindAttr(modifier) {
  return modifier.path.original === 'bind-attr';
}

function isClassBinding(pair) {
  return pair.key === 'class';
}

function classBindingToAttribute(pair) {
  var classString = pair.value.value;
  var nodes = (0, _classStringParser2.default)(classString, { spaces: true });
  var mustLoc = nodes[1];
  var size = (0, _node.sizeOfNodes)(nodes);
  var concat = void 0,
      node = void 0;

  var quoteSize = 0;
  if (nodes.length === 1) {
    node = nodes[0];
  } else {
    node = _glimmerSyntax.builders.concat(nodes);
    quoteSize = 2;
  }

  var loc = _glimmerSyntax.builders.loc(1, 0, 1 + size.line, size.column + 6 + quoteSize + 1);
  return _glimmerSyntax.builders.attr('class', node, loc);
}

function attributeBindingToAttribute(pair) {
  var attrKey = pair.key;
  var startAttr = 0;
  var equalCol = startAttr + attrKey.length;
  var startMust = equalCol + 1; // "="  1

  var value = void 0;
  if (pair.value.type === 'PathExpression') {
    value = pair.value.original;
  } else if (pair.value.type === 'StringLiteral') {
    value = pair.value.value;
  }
  var valueLength = value.length;
  var endMust = startMust + 2 + valueLength + 2;

  var node = _glimmerSyntax.builders.mustache(value);
  node.loc = _glimmerSyntax.builders.loc(1, startMust, 1, endMust);
  node.path.loc = _glimmerSyntax.builders.loc(1, startMust + 2, 1, endMust - 2);
  var newAttr = _glimmerSyntax.builders.attr(pair.key, node);
  newAttr.loc = _glimmerSyntax.builders.loc(1, startAttr, 1, endMust);
  return newAttr;
}

function removeBindAttr(modifier, node, ast) {
  var pairs = modifier.hash.pairs;
  var attrs = [];
  for (var j = 0; j < pairs.length; j++) {
    var pair = pairs[j];
    var attr = void 0;
    if (isClassBinding(pair)) {
      attr = classBindingToAttribute(pair);
    } else {
      attr = attributeBindingToAttribute(pair);
    }
    attrs.push({ pair: pair, attr: attr });
  }

  var firstAttrOffset = void 0;
  var prevAttrsOffset = { line: 0, column: 0 };
  // set locations of new attributes
  for (var i = 0; i < attrs.length; i++) {
    var _attrs$i = attrs[i],
        _attr = _attrs$i.attr,
        _pair = _attrs$i.pair;

    if (i === 0) {
      firstAttrOffset = (0, _location.locDiff)(_attr.loc, modifier.loc).start;
      (0, _node.offsetNode)(_attr, firstAttrOffset, { recursive: true, both: true });
      prevAttrsOffset = (0, _location.addOffsets)(prevAttrsOffset, (0, _node.sizeOfNodes)(_attr));
    } else {
      // find old offset to previous value
      var _attrs = attrs[i - 1],
          prevAttr = _attrs.attr,
          prevPair = _attrs.pair;

      (0, _node.offsetNode)(_attr, firstAttrOffset, { recursive: true, both: true });

      var _offset = { line: 0, column: 0 };

      var prevPairDiff = {
        line: _pair.loc.start.line - prevPair.loc.end.line,
        column: _pair.loc.start.column - prevPair.loc.end.column
      };
      _offset = (0, _location.addOffsets)(_offset, prevPairDiff);
      _offset = (0, _location.addOffsets)(_offset, prevAttrsOffset);
      (0, _node.offsetNode)(_attr, _offset, { recursive: true, both: false });

      prevAttrsOffset = (0, _location.addOffsets)(prevAttrsOffset, (0, _node.sizeOfNodes)(_attr));
      prevAttrsOffset = (0, _location.addOffsets)(prevAttrsOffset, prevPairDiff);
    }
  }

  (0, _node.removeNode)(modifier, ast);

  var firstAttr = attrs[0].attr;
  var lastAttr = attrs[attrs.length - 1].attr;
  var startingAt = modifier.loc.start;
  var offset = prevAttrsOffset;
  (0, _node.offsetNode)(ast, offset, { recursive: true, startingAt: startingAt });

  var bindAttrIndex = (0, _node.nodeIndex)(modifier, node.attributes);
  attrs.forEach(function (a, i) {
    return node.attributes.splice(bindAttrIndex + i, 0, a.attr);
  });
}

function convertBindAttr(ast) {
  var walker = new _glimmerSyntax.Walker(ast);
  var flattened = ast;

  walker.visit(ast, function (node) {
    if (node.type === 'ElementNode' && node.modifiers) {
      var modifiers = _underscore2.default.clone(node.modifiers);
      for (var i = 0; i < modifiers.length; i++) {
        var modifier = modifiers[i];
        if (isBindAttr(modifier)) {
          removeBindAttr(modifier, node, ast);
        }
      }
    }
  });

  return ast;
}

exports.attributeBindingToAttribute = attributeBindingToAttribute;
exports.removeBindAttr = removeBindAttr;