'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = rewrite;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _broccoli = require('broccoli');

var _broccoli2 = _interopRequireDefault(_broccoli);

var _broccoliFunnel = require('broccoli-funnel');

var _broccoliFunnel2 = _interopRequireDefault(_broccoliFunnel);

var _broccoliPersistentFilter = require('broccoli-persistent-filter');

var _broccoliPersistentFilter2 = _interopRequireDefault(_broccoliPersistentFilter);

var _process = require('./process');

var _process2 = _interopRequireDefault(_process);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Rewrite(inputNode, options) {
  options = options || {};
  _broccoliPersistentFilter2.default.call(this, inputNode, {
    persist: true
  });
  this.options = options;
}
Rewrite.prototype = Object.create(_broccoliPersistentFilter2.default.prototype);
Rewrite.prototype.constructor = Rewrite;

Rewrite.prototype.baseDir = function () {
  return _path2.default.join(__dirname, '../');
};

Rewrite.prototype.processString = function (template, srcFile) {
  console.log(srcFile);
  try {
    return (0, _process2.default)(template, this.options);
  } catch (e) {
    console.log(template);
    console.log({ location: e.location });
    console.log(e.trace);
    // throw e;
    return template;
  }
};

function build(tree, outputPath) {
  var builder = new _broccoli2.default.Builder(tree);
  return builder.build().then(function () {
    _fsExtra2.default.copySync(builder.outputPath, outputPath);
  }).finally(function () {
    return builder.cleanup();
  }).catch(function (err) {
    console.error(err.stack);
    _process2.default.exit(1);
  });
}

function rewrite(path, options) {
  var templates = new _broccoliFunnel2.default(path, {
    include: ['**/*.hbs'],
    exclude: ['**/node_modules/**/*', '**/bower_components/**/*']
  });

  templates = new Rewrite(templates, options);

  build(templates, options.outputPath);
}