'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sizeOfNodes = exports.enumerate = exports.removeNode = exports.offsetNode = exports.nodeIndex = exports.nodesAfter = exports.nodesBefore = exports.sortNodes = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _glimmerSyntax = require('glimmer-engine/dist/node_modules/glimmer-syntax');

var _location = require('./location');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function nodeIndex(node) {
  for (var _len = arguments.length, lists = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    lists[_key - 1] = arguments[_key];
  }

  var nodes = sortNodes.apply(undefined, [node].concat(lists));
  return nodes.indexOf(node);
}

function sortNodes() {
  var allNodes = [];

  for (var _len2 = arguments.length, nodes = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    nodes[_key2] = arguments[_key2];
  }

  nodes.forEach(function (n) {
    return allNodes = allNodes.concat(n);
  });
  return _underscore2.default.uniq(allNodes).sort(function (a, b) {
    var lineDiff = a.loc.start.line - b.loc.start.line;
    var colDiff = a.loc.start.column - b.loc.start.column;
    return lineDiff === 0 && colDiff > 0 || lineDiff > 0;
  });
}

function nodesBefore(before) {
  for (var _len3 = arguments.length, nodes = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    nodes[_key3 - 1] = arguments[_key3];
  }

  nodes = nodes.reduce(function (acc, n) {
    return acc.concat(n);
  }, []);
  var sorted = sameLine(before, nodes);
  var index = sorted.indexOf(before);
  var start = index;
  var currentLine = before.loc.start.line;
  for (var i = start; i >= 0; i--) {
    var endLine = sorted[i].loc.end.line;
    if (endLine === currentLine) {
      start = i;
    } else {
      break;
    }
  }
  return sorted.slice(start, index);
}

function nodesAfter(after) {
  for (var _len4 = arguments.length, nodes = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
    nodes[_key4 - 1] = arguments[_key4];
  }

  nodes = nodes.reduce(function (acc, n) {
    return acc.concat(n);
  }, []);
  var sorted = sameLine(after, nodes);
  var index = sorted.indexOf(after);
  var stop = index;
  var currentLine = after.loc.end.line;
  for (var i = stop; i < sorted.length; i++) {
    var endLine = sorted[i].loc.end.line;
    if (endLine === currentLine) {
      stop = i;
    } else {
      break;
    }
  }
  return sorted.slice(index + 1, stop + 1);
}

function enumerate(node, func, options) {
  if (!node) {
    return node;
  }

  var _ref = options || {},
      parent = _ref.parent,
      property = _ref.property;

  if (node.type) {
    func(node, options || {});
  }
  Object.entries(node).forEach(function (_ref2, i) {
    var _ref3 = _slicedToArray(_ref2, 2),
        key = _ref3[0],
        val = _ref3[1];

    if (val && val.type) {
      enumerate(val, func, { parent: node, property: key, isArray: false });
    } else if (Array.isArray(val)) {
      var nodes = _underscore2.default.clone(val);
      nodes.forEach(function (n, index) {
        return enumerate(n, func, {
          parent: node, property: key, isArray: true, index: index
        });
      });
    }
  });
  return node;
}

function _offsetNodeRecursive(node, offset, options) {
  if (!node) {
    return node;
  }
  if (node.loc) {
    _offsetNode(node, offset, options);
  }
  Object.entries(node).forEach(function (_ref4, i) {
    var _ref5 = _slicedToArray(_ref4, 2),
        key = _ref5[0],
        val = _ref5[1];

    if (val && val.type) {
      _offsetNodeRecursive(val, offset, options);
    } else if (Array.isArray(val)) {
      val.forEach(function (v) {
        return _offsetNodeRecursive(v, offset, options);
      });
    }
  });
  return node;
}

function offsetNode(node, offset, options) {
  if (Array.isArray(node)) {
    return node.map(function (n) {
      return offsetNode(n, offset, options);
    });
  }
  if (!node || !node.loc) {
    throw new Error('offsetNode must be given a node with a loc');
  }
  if (options && options.recursive) {
    return _offsetNodeRecursive(node, offset, options);
  } else {
    return _offsetNode(node, offset, options);
  }
}

function _offsetNode(node, offset, options) {
  if (!node || !node.loc) {
    throw new Error('_offsetNode must be given a node with a loc');
  }
  options = options || {};
  var loc = node.loc;
  var _options = options,
      startingAt = _options.startingAt;


  if (offset.line && !options.both) {
    offset.column = 0;
  }

  var diff = _glimmerSyntax.builders.loc(offset.line, offset.column, offset.line, offset.column);

  if (startingAt) {
    // entire node is before start
    if ((0, _location.posBefore)(loc.end, startingAt)) {
      diff = null;
    }

    // on line below start
    if (loc.start.line > startingAt.line) {
      diff.start.column = 0;
      diff.end.column = 0;
    }

    if (loc.end.line > startingAt.line) {
      if (loc.start.line === startingAt.line) {
        // starts on same line after but ends on lower line
        // shift start but not end
        diff.end.column = 0;
      } else {
        // node ends below line than starting line
        diff.end.column = 0;
      }
    }

    // node starts before start but ends after
    if ((0, _location.posBefore)(loc.start, startingAt) && (0, _location.posAfter)(loc.end, startingAt)) {
      diff.start.line = 0;
      diff.start.column = 0;
    }
  }

  if (!diff) {
    return node;
  }

  loc = _glimmerSyntax.builders.loc(loc);
  loc = (0, _location.locAdd)(loc, diff);
  node.loc = loc;
  return node;
}

function sameLine(node) {
  for (var _len5 = arguments.length, nodes = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
    nodes[_key5 - 1] = arguments[_key5];
  }

  nodes = nodes.reduce(function (acc, n) {
    return acc.concat(n);
  }, []);
  return sortNodes([node], nodes);
}

function removeNode(target, ast) {
  enumerate(ast, function (node, _ref6) {
    var parent = _ref6.parent,
        property = _ref6.property,
        isArray = _ref6.isArray,
        index = _ref6.index;

    if (target === node) {
      if (isArray) {
        parent[property].splice(index, 1);
      } else {
        delete parent[property];
      }
    }
  });

  var startingAt = target.loc.start;
  var span = (0, _location.locSpan)(target.loc);
  span.line = -span.line;
  span.column = -span.column;
  var after = nodesAfter(target, ast);
  var before = nodesAfter(target, ast);
  offsetNode(ast, span, { recursive: true, startingAt: startingAt });
  if (after.length === 0 && before.length === 0) {
    // TODO: delete empty line
  }

  return ast;
}

function sizeOfNodes(nodes) {
  var minLine = null,
      maxLine = null,
      minCol = null,
      maxCol = null;
  enumerate({ nodes: nodes }, function (n) {
    if (n.loc) {
      var value = n.value || n.chars || '';
      var _n$loc = n.loc,
          _n$loc$start = _n$loc.start,
          startLine = _n$loc$start.line,
          startCol = _n$loc$start.column,
          _n$loc$end = _n$loc.end,
          endLine = _n$loc$end.line,
          endCol = _n$loc$end.column;

      if (minLine === null || minLine > startLine) {
        minLine = startLine;
      }
      if (maxLine === null || maxLine < endLine) {
        maxLine = endLine;
      }
      if (minCol === null || minCol > startCol) {
        minCol = startCol;
      }
      if (maxCol === null || maxCol < endCol) {
        maxCol = endCol;
      }
    }
  }, { recursive: true });
  return _glimmerSyntax.builders.loc(0, 0, maxLine - minLine, maxCol - minCol).end;
}

exports.sortNodes = sortNodes;
exports.nodesBefore = nodesBefore;
exports.nodesAfter = nodesAfter;
exports.nodeIndex = nodeIndex;
exports.offsetNode = offsetNode;
exports.removeNode = removeNode;
exports.enumerate = enumerate;
exports.sizeOfNodes = sizeOfNodes;